bl_info = {
	"name": "Playback Wait/Stop Loop",
	"description": "Wait for user to begin playback / Disable playback loop",
	"author": "Your Name",
	"version": (1, 0, 0),
	"blender": (2, 79, 3),
	"location": "Timeline",
	"wiki_url": "",
	"category": "Animation" }

import bpy

bpy.types.Screen.animation_stop = bpy.props.BoolProperty(default=False);
bpy.types.Screen.animation_wait = bpy.props.BoolProperty(default=False);
bpy.types.Screen.animation_mode = bpy.props.StringProperty(default='');


class toggle_playback_loop(bpy.types.Operator):
	bl_idname = "my_operator.toggle_playback_loop"
	bl_label = "Toggle Looping"
	bl_description = "Enable/Disable Animation Loop"
	#bl_options = {"REGISTER"}

	@classmethod
	def poll(cls, context):
		return True

	def execute(self, context):
		bpy.context.screen.animation_stop = not bpy.context.screen.animation_stop;
		return {"FINISHED"}

		
class wait_for_input(bpy.types.Operator):
	bl_idname = "screen.animation_play_wait"
	bl_label = "Wait for Input (loop)"
	bl_description = "Begin playback when key is pressed"
	#bl_options = {"REGISTER"}
	
	timer = bpy.props.IntProperty(default = 0);
	used = bpy.props.BoolProperty(default = False);
	
	@classmethod
	def poll(cls, context):
		return True

	def invoke(self, context, event):
		context.window_manager.modal_handler_add(self)
		if bpy.context.screen.animation_wait and (not bpy.context.screen.is_animation_playing):	
			self.report({'INFO'}, 'Playback Waiting for user input')
		if not 'overwrite_keys' in str(bpy.app.handlers.frame_change_pre):
			def overwrite_keys(scene):
				if bpy.context.screen.animation_mode != '':
					if bpy.context.mode == 'OBJECT':
						if context.object:
							selected = bpy.context.selected_objects;
							for src in selected:
								if src.animation_data and src.animation_data.action:
									for curve in src.animation_data.action.fcurves:
										path = curve.data_path
										if any(((bpy.context.screen.animation_mode in curve.data_path) 
											,(bpy.context.screen.animation_mode=='all' and path in ('location','rotation_quaternion','rotation_euler','scale'))
											,(bpy.context.screen.animation_mode=='locrot' and path in ('location','rotation_quaternion','rotation_euler'))
											,(bpy.context.screen.animation_mode=='locscale' and path in ('location','scale'))
											)):
											if bpy.context.screen.is_animation_playing:		curve.mute = True;
											else:											curve.mute = False;
					if bpy.context.mode == 'POSE':
						if bpy.context.active_object and bpy.context.selected_pose_bones:
							obj = bpy.context.active_object;
							selected = bpy.context.selected_pose_bones;
							if obj.animation_data and obj.animation_data.action:
								for curve in obj.animation_data.action.fcurves:
									path = curve.data_path;
									for src in selected:
										if ('pose.bones["'+src.name+'"]') in path:
											if any((
												(bpy.context.screen.animation_mode in path.rsplit('"].',1)[1])
												,(bpy.context.screen.animation_mode=='all' and path.rsplit('"].',1)[1] in ('location','rotation_quaternion','rotation_euler','scale'))
												,(bpy.context.screen.animation_mode=='locrot' and path.rsplit('"].',1)[1] in ('location','rotation_quaternion','rotation_euler'))
												,(bpy.context.screen.animation_mode=='locscale' and path.rsplit('"].',1)[1] in ('location','scale'))
												)):
												if bpy.context.screen.is_animation_playing:		curve.mute = True;
											else:											curve.mute = False;
				if not bpy.context.screen.is_animation_playing: bpy.context.screen.animation_mode = ''
			bpy.app.handlers.frame_change_pre.append(overwrite_keys)
		if not 'update_keys' in str(bpy.app.handlers.frame_change_post):
			def update_keys(scene):
				if bpy.context.screen.animation_mode != '':
					if bpy.context.mode == 'OBJECT':
						if context.object:
							selected = bpy.context.selected_objects;
							for src in selected:
								if src.animation_data and src.animation_data.action:
									for curve in src.animation_data.action.fcurves:
										path = curve.data_path
										if any(((bpy.context.screen.animation_mode in curve.data_path) 
											,(bpy.context.screen.animation_mode=='all' and path in ('location','rotation_quaternion','rotation_euler','scale'))
											,(bpy.context.screen.animation_mode=='locrot' and path in ('location','rotation_quaternion','rotation_euler'))
											,(bpy.context.screen.animation_mode=='locscale' and path in ('location','scale'))
											)):
											curve.mute = False;
					if bpy.context.mode == 'POSE':
						if bpy.context.active_object and bpy.context.selected_pose_bones:
							obj = bpy.context.active_object;
							selected = bpy.context.selected_pose_bones;
							if obj.animation_data and obj.animation_data.action:
								for curve in obj.animation_data.action.fcurves:
									path = curve.data_path;
									for src in selected:
										if ('pose.bones["'+src.name+'"]') in path:
											if any((
												(bpy.context.screen.animation_mode in path.rsplit('"].',1)[1])
												,(bpy.context.screen.animation_mode=='all' and path.rsplit('"].',1)[1] in ('location','rotation_quaternion','rotation_euler','scale'))
												,(bpy.context.screen.animation_mode=='locrot' and path.rsplit('"].',1)[1] in ('location','rotation_quaternion','rotation_euler'))
												,(bpy.context.screen.animation_mode=='locscale' and path.rsplit('"].',1)[1] in ('location','scale'))
												)):
												curve.mute = False; 
				if not bpy.context.screen.is_animation_playing: bpy.context.screen.animation_mode = ''
			bpy.app.handlers.frame_change_post.append(update_keys)
		if not 'stop_playback' in str(bpy.app.handlers.frame_change_pre):
			def stop_playback(scene): #stop at last frame
				if bpy.context.screen.animation_stop:
					if (not scene.use_preview_range and scene.frame_current == scene.frame_end) or (scene.use_preview_range and scene.frame_current == scene.frame_preview_end):		bpy.ops.screen.animation_cancel(restore_frame=False)
			bpy.app.handlers.frame_change_pre.append(stop_playback)
		return {"RUNNING_MODAL"}

	def modal(self, context, event):
		
	
		if bpy.context.screen.is_animation_playing:
			#bpy.ops.screen.animation_cancel(restore_frame=True)#
			bpy.ops.screen.animation_play('INVOKE_DEFAULT'); 
			return{"FINISHED"}
		if bpy.context.screen.animation_wait:
			if self.timer == 0: bpy.ops.screen.animation_cancel(restore_frame=False);

			if event.type in ('SPACE','Just Play'):			bpy.ops.screen.animation_play('INVOKE_DEFAULT'); self.timer = 0; return{"FINISHED"}
			
			if event.type in('G','R','S','LEFTMOUSE'):
				pivot = bpy.context.space_data.pivot_point;
				if event.type in ('G',''): 
					bpy.context.screen.animation_mode = 'location';
					if bpy.context.mode == 'POSE':
						bone = bpy.context.active_pose_bone;
						if bone.bone.use_connect:	bpy.context.screen.animation_mode = 'all';
				if event.type in ('R',''):
					bpy.context.screen.animation_mode = 'rotation';
					if pivot == 'INDIVIDUAL_ORIGINS': bpy.context.screen.animation_mode = 'rotation';
					if pivot == 'CURSOR': bpy.context.screen.animation_mode = 'locrot';
					if pivot in ('ACTIVE_ELEMENT','BOUNDING_BOX_CENTER','MEDIAN_POINT'):
						if bpy.context.mode == 'POSE' and len(bpy.context.selected_pose_bones) > 1: bpy.context.screen.animation_mode = 'locrot'
						if bpy.context.mode == 'OBJECT' and len(bpy.context.selected_objects) > 1: bpy.context.screen.animation_mode = 'locrot';
				if event.type in ('S',''):
					bpy.context.screen.animation_mode = 'scale';
					if pivot == 'INDIVIDUAL_ORIGINS': bpy.context.screen.animation_mode = 'scale';
					if pivot == 'CURSOR': bpy.context.screen.animation_mode = 'locscale';
					if pivot in ('ACTIVE_ELEMENT','BOUNDING_BOX_CENTER','MEDIAN_POINT'):
						if bpy.context.mode == 'POSE' and len(bpy.context.selected_pose_bones) > 1: bpy.context.screen.animation_mode = 'locscale'
						if bpy.context.mode == 'OBJECT' and len(bpy.context.selected_objects) > 1: bpy.context.screen.animation_mode = 'locscale';
				if event.type in ('LEFTMOUSE',''): bpy.context.screen.animation_mode = 'all';
				bpy.ops.screen.animation_play('INVOKE_DEFAULT'); 
				self.timer = 0; return{"PASS_THROUGH","FINISHED"}
			if event.type in {"ESC",'Stop'}:
				self.timer = 0; return {'PASS_THROUGH',"CANCELLED"}
			#if event.type in('A','D', 'MIDDLEMOUSE','RIGHTMOUSE','Ignored'):
			
			#log unknown types
			#elif not event.type in ('MOUSEMOVE','TIMER','INBETWEEN_MOUSE_MOVE','',''): print('type = '+str(event.type))
			
			else: self.timer += 1; return{"PASS_THROUGH"} #Don't block keys
		
		else: bpy.ops.screen.animation_play('INVOKE_DEFAULT'); return{"FINISHED"}
		
		return {"RUNNING_MODAL"}

class overwrite_keys(bpy.types.Operator):
	bl_idname = "screen.overwrite_keys"
	bl_label = "Overwrite Keys"
	bl_description = "Remove Keyframes during playback"
	#bl_options = {"REGISTER"}
	
	
	@classmethod
	def poll(cls, context):
		return True

	def invoke(self, context, event):
		context.window_manager.modal_handler_add(self)
		self.report({'INFO'}, 'Running Overwrite Keys')
		return {"RUNNING_MODAL"}

	def modal(self, context, event):
		if context.mode == 'POSE':
			selected = context.selected_pose_bones;
		if context.mode == 'OBJECT':
			selected = context.selected_objects;
		for src in selected:
			if src.animation_data and src.animation_data.action:
				for curve in src.animation_data.action.fcurves:
					keys = curve.keyframe_points;
					for key in keys:
						if (round(abs(key.co[0])) - round(abs(bpy.context.scene.frame_current))) == 0: #difference between numbers
							keys.remove(key, fast = False); #Remove key and readjust curve
							
		return {"RUNNING_MODAL"}

class toggle_playback_loop_panel(bpy.types.Header):
	bl_idname = "toggle_playback_loop_panel"
	#bl_label = "Toggle Looping panel"
	bl_space_type = "TIMELINE"
	bl_region_type = "HEADER"
	#bl_category = "category"

	def draw(self, context):
		layout = self.layout
		row = layout.row(align=True)
		stop = bpy.context.screen.animation_stop
		wait = bpy.context.screen.animation_wait
		row.prop(bpy.context.screen,'animation_wait','Wait' if wait else 'Play' ,icon='PAUSE' if wait else 'PLAY')
		row.prop(bpy.context.screen,'animation_stop',text='Once' if stop else 'Loop',icon='TRIA_RIGHT_BAR' if stop else 'FILE_REFRESH');
		#row.operator('my_operator.toggle_playback_loop','Loop' if loop else 'Once',icon='FILE_REFRESH' if loop else 'PAUSE', emboss=False)

def register():
	bpy.utils.register_module(__name__)

def unregister():
	bpy.utils.unregister_module(__name__)

if __name__ == "__main__":
	register()

